/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define BASECONFIG_C_
#include "baseconfig.h"

struct stanza *first_stanza, *last_stanza, *current_stanza;

static char *label, *command, *key;
static int multiple, text;

static cl_var_t tab[] ={
	{ "label",   	 NULL,	CL_STRING,  &label,	0,	NULL },
	{ "multiple",    NULL,	CL_BOOLEAN, &multiple,	0,	NULL },
	{ "command",	 NULL,	CL_STRING,  &command,	0,	NULL },
	{ "text",	 NULL,	CL_BOOLEAN, &text,	0,	NULL },
	{ "key", 	 NULL,	CL_STRING,  &key,	0,	NULL },
	{ NULL, NULL,  0, NULL, 0, NULL}
};

void scroll_stanzas(long n)
{
	for (current_stanza = first_stanza; n; n--)
		current_stanza = current_stanza->next;
}

static char *set_char(char *src, char **dest)
{
	if (src)
		*dest = strdup(src);
	else
		*dest = NULL;
		
	return *dest;
}

static void set_varibles()
{
	command = label = key = NULL;	
	multiple = text = 0;
}

static int add_stanza(char *name)
{
	struct stanza *s = xmalloc(sizeof(struct stanza));
		
	if ((set_char(name, &s->name)) == NULL ||
		set_char(label, &s->label) == NULL)
		return 0;
		
	set_char(command, &s->command);
	s->key = (key) ? *key: 0;
	s->multiple = multiple;		
	s->text = text;
	
	if (first_stanza == NULL)
		current_stanza = first_stanza = last_stanza = s;
	else {
		last_stanza->next = s;
		last_stanza = s;
	}
	
	s->next = NULL;
	set_varibles();	
	return 1;
}


int read_baseconfig(char *name) 
{
	char **ar, **p;
	cl_ulist_t *unknows = NULL;
	cl_var_t *t = tab;
	cl_file_t *handle;
	
	if (!name) 
		return 0;
	
	if (!(handle = cl_openconf(name, "lr")))
		if (!(handle = cl_openconf(name, "r"))) 
			return 0;
	
	if (!(ar = cl_build_stanza_array("*",handle))) 
		return 0;
		
	msgf("read in base config file:  '%s'", name);
	
	for (p = ar; *p && !cl_getstanza(handle, t, *p, 0, &unknows); p++) {
		if (*p == NULL)
			continue;
		if (add_stanza(*p) == 0)
			return 0;
	}
		
	cl_closeconf(handle);
	return 1;
}

struct stanza *find_stanza_by_name(char *name)
{
	struct stanza *s = first_stanza;
	for (; s; s = s->next)
		if (!strcmp(s->name, name))
			break;
	return s;
}

void clear_allenv()
{
	struct stanza *s = first_stanza;
	for (; s; s = s->next)
		setenv(s->name, "", 1);
}

void free_stanzas(void)
{
	struct stanza *sj;
	for (current_stanza = first_stanza; current_stanza;) {
		sj = current_stanza;
		current_stanza = sj->next;
		xfree(sj->label);
		xfree(sj->name);
		if (sj->command)
			xfree(sj->command);
		xfree(sj);
	}
}
