#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include "tobj.h"
#include "tsys.h"
#include "twin.h"
#include "tfamily.h"
#include "tinput.h"
#include "tlist.h"
#include "tlabel.h"
#include "tbutton.h"

void tset(struct tobj *obj, long attr, long data)
{
	struct tobj_set tos;
	tos.attr = attr;
	tos.data = data;
	tobj_domethod(obj, TOBJ_SET, TPDATA(&tos));
}
void tmset(struct tobj *obj, ...)
{
	struct tobj_set tos;
	long attr;
	va_list ap;

	va_start(ap, obj);

	while ((attr = va_arg(ap, long))) {

		tos.attr = attr;
		tos.data = va_arg(ap, long);
		tobj_domethod(obj, TOBJ_SET, TPDATA(&tos));
	}
	va_end(ap);
}
void tget(struct tobj *obj, long attr, long *data)
{
	struct tobj_get tog;
	tog.attr = attr;
	tog.data = data;
	tobj_domethod(obj, TOBJ_GET, TPDATA(&tog));
}
long txget(struct tobj *obj, long attr)
{
	struct tobj_get tog;
	long data;

	tog.attr = attr;
	tog.data = &data;
	tobj_domethod(obj, TOBJ_GET, TPDATA(&tog));

	return data;
}

char *treq_list(struct tobj *parent, unsigned int x, unsigned int y,
		unsigned int w, unsigned int h, char *title, char **labels,
		long defval)
{
	struct tobj *win, *list, *active;
	int quit = 0;
	unsigned long len;
	signed long ret = -1;
	char errmsg[] = "ERROR: While opening list requester.\n";

	if (!w) {
		unsigned long i;
		for (i = 0; labels[i]; i++) {
			len = strlen(labels[i]);
			if (w < len)
				w = len;
		}
		w += 2;
	}
	if (title){
		len = strlen(title);
		if (w < len + 4)
			w = len + 4;
	}

	if (!h) {
		while (labels[h])
			h++;
		h += 3;
	}
	if (h < 5)
		h = 5;

	if (!x)
		x = parent->w / 2 - w / 2;
	
	if (w + x + tobj_getx(parent)> t_screencols())
		x = t_screencols() - w - tobj_getx(parent);

	if (!y)
		y = parent->w / 2 - h / 2;
	
	if (h + y + tobj_gety(parent) > t_screenrows())
		y = t_screenrows() - h - tobj_gety(parent);
	
	active = tfamily_activechild( parent );

	if (!(win = twin_init(parent, x, y, w, h, title)))
		tsys_error_exit(errmsg);


	if (!(list = tlist_init(win, 1, 1, w - 2, h - 2, labels)))
		tsys_error_exit(errmsg);

	tset(list, TLIST_ACTIVE, defval);
	tobj_domethod(win, TOBJ_ACTIVATE, 0);
	while (!quit) {		/* do loop unless quit == 1 */
		struct tevent *e;
		if (( e = tsys_process(win) )) {

			if (e->data == EVENT_QUIT)
				quit = 1;

			else if (e->data == EVENT_TWIN_CLOSE
				 && e->sender == win)
				quit = 1;
			else if (e->data == EVENT_TLIST_ACCEPT
				 && e->sender == list) {
				ret = txget(list, TLIST_ACTIVE);
				quit = 1;
			} else if (e->data == EVENT_TLIST_REJECT
				   && e->sender == list) {
				quit = 1;
			} else if (e->data == K_ESC)
				quit = 1;

			tevent_free(e);
		}

	}

	/* freeing window and it's childs */
	tobj_free(win);
	
	tobj_domethod(active, TOBJ_ACTIVATE, 0);

	if (ret != -1)
		return labels[ret];
	else
		return NULL;
}
char *treq_string(struct tobj *parent, char *title, char *label,
		char *defval, char *buffer, unsigned long buflen)
{
	struct tobj *win, *input, *button, *active;
	int quit = 0;
	int len1, len2, x, inputlen = 20;
	char errmsg[] = "ERROR: While opening string requester.\n";

	len1 = strlen(title);
	len2 = strlen(label);

	x = MAX(len1 + 4, len2 + inputlen + 3);
	inputlen = x - len2 - 3;

	active = tfamily_activechild( parent );
	
	if (!
	    (win =
	     twin_init(parent, parent->w / 2 - x / 2, parent->h / 2 - 2, x,
		       5, title)))
		tsys_error_exit(errmsg);

	if (!(tlinelabel_init(win, 1, 1, label)))
		tsys_error_exit(errmsg);

	if (!(input = tinput_init(win, len2 + 2, 1, inputlen, defval, buflen)))
		tsys_error_exit(errmsg);

	if (!(button = tbutton_init(win, x / 2 - 3, 3, 6, "OK")))
		tsys_error_exit(errmsg);

	tobj_domethod(win, TOBJ_ACTIVATE, 0);
	while (!quit) {		/* do loop unless quit == 1 */
		struct tevent *e;
		if (( e = tsys_process(win) )) {

			if (e->data == EVENT_QUIT)
				quit = 1;

			else if (e->data == EVENT_TWIN_CLOSE
				 && e->sender == win)
				quit = 1;
			else if (e->sender == button &&
				 e->data == EVENT_TBUTTON_PRESSED) {
				quit = 1;
			} else if (e->data == K_ESC)
				quit = 1;

			tevent_free(e);
		}

	}

	strcpy( buffer, (char *) txget(input, TINPUT_BUFFER_CURRENT));
	/* freeing window and it's childs */
	tobj_free(win);
	tobj_domethod(active, TOBJ_ACTIVATE, 0);

	return buffer;
}
long treq_integer(struct tobj *parent, char *title, char *label, long defval)
{
	struct tobj *win, *input, *button, *active;
	int quit = 0;
	int len1, len2, x, inputlen = 10;
	long ret;
	char s[11], errmsg[] = "ERROR: While opening integer requester.\n";

	len1 = strlen(title);
	len2 = strlen(label);

	x = MAX(len1 + 4, len2 + inputlen + 3);
	inputlen = x - len2 - 3;

	active = tfamily_activechild( parent );

	if (!
	    (win =
	     twin_init(parent, parent->w / 2 - x / 2, parent->h / 2 - 2, x,
		       5, title)))
		tsys_error_exit(errmsg);

	if (!(tlinelabel_init(win, 1, 1, label)))
		tsys_error_exit(errmsg);

	sprintf(s, "%li", defval);
	if (!(input = tinput_init(win, len2 + 2, 1, inputlen, s, 10)))
		tsys_error_exit(errmsg);

	tset(input, TINPUT_ACCEPT_CHARS, TPDATA("0123456789"));

	if (!(button = tbutton_init(win, x / 2 - 3, 3, 6, "OK")))
		tsys_error_exit(errmsg);

	tobj_domethod(win, TOBJ_ACTIVATE, 0);
	while (!quit) {		/* do loop unless quit == 1 */
		struct tevent *e;
		if (( e = tsys_process(win) )) {

			if (e->data == EVENT_QUIT)
				quit = 1;

			else if (e->data == EVENT_TWIN_CLOSE
				 && e->sender == win)
				quit = 1;
			else if (e->sender == button &&
				 e->data == EVENT_TBUTTON_PRESSED) {
				quit = 1;
			} else if (e->data == K_ESC)
				quit = 1;

			tevent_free(e);
		}

	}

	ret = atol((char *) txget(input, TINPUT_BUFFER_CURRENT));
	/* freeing window and it's childs */
	tobj_free(win);
	tobj_domethod(active, TOBJ_ACTIVATE, 0);

	return ret;
}

void treq_info(struct tobj *parent, char *title, char *label)
{
	struct tobj *win, *button, *active;
	int quit = 0;
	int len1, len2, x;
	char errmsg[] = "ERROR: While opening info requester.\n";

	len1 = strlen(title);
	len2 = strlen(label);

	x = MAX(len1 + 4, len2 + 4);
	
	active = tfamily_activechild( parent );

	if (!
	    (win =
	     twin_init(parent, parent->w / 2 - x / 2, parent->h / 2 - 2, x,
		       5, title)))
		tsys_error_exit(errmsg);

	if (!(tlinelabel_init(win, 1, 1, label)))
		tsys_error_exit(errmsg);

	if (!(button = tbutton_init(win, x / 2 - 3, 3, 6, "OK")))
		tsys_error_exit(errmsg);

	tobj_domethod(win, TOBJ_ACTIVATE, 0);
	while (!quit) {		/* do loop unless quit == 1 */
		struct tevent *e;
		if ((e = tsys_process(win))) {
			if (e->data == EVENT_QUIT)
				quit = 1;

			else if (e->data == EVENT_TWIN_CLOSE
				 && e->sender == win)
				quit = 1;
			else if (e->sender == button &&
				 e->data == EVENT_TBUTTON_PRESSED) {
				quit = 1;
			} else if (e->data == K_ESC)
				quit = 1;

			tevent_free(e);
		}

	}

	/* freeing window and it's childs */
	tobj_free(win);
	tobj_domethod(active, TOBJ_ACTIVATE, 0);

	return;
}
