#include "tsys.h"
#include "tobj.h"
#include "tfamily.h"
#include "twin.h"
/* defines of events and methods */
#include "ttoolbar.h"
#include "ttool.h"
#include <string.h>

#include "prvclass/ttoolbar.h"

int add_tool(struct tobj *obj,notifyfnc fnc, int key, char *title, long s)
{
	struct ttoolbar_entry te;
	te.fnc = fnc;
	te.key = key;
	te.title = title;
	te.state =s;
	if (!tobj_domethod(obj, TTOOLBAR_ENTRY_ADD , TPDATA(&te)))
		return 0;
	return 1;
}


void ttoolbar_dorefresh(struct tobj *obj)
{
	struct ttoolbar *toolbar;
	struct tobj *child;
	struct ttool *tool;
	unsigned int i, x = obj->x, y = obj->y;

	toolbar = (struct ttoolbar *) obj->data;
	t_textbackground(obj->bgcolor);

	for (child = toolbar->tool, i = 1; child; child = child->next, i+=2) {
		tool = (struct ttool*)child->data; 
		if (tool->state == TTOOL_HIDDEN) {
			i -= 2;
			continue;
		}
			
		t_gotoxy(x + i + 1,  y);

		t_textcolor(obj->fgcolor);

		
		i += strlen(tool->title) + 2;
		
		if (i+2 > t_screen_width)
			break;
	
		t_textcolor(obj->bgcolor2);
		if (!tool->state)
			t_putchar('[');
		else
			t_putchar(' ');
			
		if (strchr(tool->title, tool->key)) {
			if (!tool->state)
				t_lcputs(tool->title,tool->key, 
					obj->fgcolor, obj->fgcolor2);
			else 
				t_lcputs(tool->title,tool->key, 
					obj->fgcolor2, obj->fgcolor);
			
		} else {
			t_textcolor(obj->fgcolor2);
			t_putchar(tool->key);
			t_textcolor(obj->fgcolor);
			t_putchar(':');
			t_cputs(tool->title);
			i+= 2;
		}
		t_textcolor(obj->bgcolor2);
		if (!tool->state)
			t_putchar(']');
		else
			t_putchar(' ');
	}
}


static int ttoolbar_set(struct tobj *obj, struct tobj_set *tos)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;

	if (toolbar->family_fnc.domethod && 
		(*toolbar->family_fnc.domethod)(obj, TOBJ_SET, TPDATA(tos)))
		return 1;
	else
		return 0;
}

static int ttoolbar_get(struct tobj *obj, struct tobj_get *tog)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;
	if (toolbar->family_fnc.domethod && 
		(*toolbar->family_fnc.domethod)(obj, TOBJ_GET, TPDATA(tog)))
		return 1;
	else
		return 0;

}

static int ttoolbar_entry_add(struct tobj *obj, struct ttoolbar_entry *entry)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;
	struct tobj *child = toolbar->tool, *tool;
	
	if (!(tool = ttool_init(obj, entry->fnc, entry->key, 
			entry->title, entry->state)))
		return 0;
		
	if (!child)
			toolbar->tool = tool;
	else {
		for (child = toolbar->tool; child; child = child->next) 
			if (child->next == NULL) 
				break;
		child->next = tool;
		child->next->next = NULL;
		// why that ? ttoolbar is not compatibily with tsystem in 100%
	}			
	return 1;
}


int ttoolbar_domethod(struct tobj *obj, long method, long data)
{
	switch (method) {
	case TOBJ_SET:
		return ttoolbar_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return ttoolbar_get(obj, (struct tobj_get *) data);
	case TTOOLBAR_ENTRY_ADD:
		if (!(ttoolbar_entry_add(obj, (struct ttoolbar_entry*) data)))
			tsys_error_exit(
				"ERROR: While changing wordfile in tcmenu.\n");
		break;
	}
	return 1;
}

int ttoolbar_doevent(struct tobj *obj, int keycode)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;
	struct tobj *child;

	for (child = toolbar->tool; child; child = child->next) 
		if (ttool_doevent(child, keycode)) {
			return 1;
		}
	return 0;
}

void ttoolbar_cleanup(struct tobj *obj)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;
	struct tobj *jmp = toolbar->tool;
	struct tobj *j2 = jmp;
	
	while (jmp) {
		jmp = j2->next;
		ttool_free(j2);
		j2 = jmp;
	}
	
}

void ttoolbar_free(struct tobj *obj)
{
	ttoolbar_cleanup(obj);
}

int ttoolbar_setup(struct tobj *obj, struct tobj_fnc *fnc)
{
	struct ttoolbar *toolbar = (struct ttoolbar *) obj->data;

	toolbar->tool = 0;
	if (!( tfamily_setup( obj , &toolbar->family_fnc ) ))
		return 0;

	fnc->free = ttoolbar_free;
	fnc->garbage = toolbar->family_fnc.garbage;
	fnc->dorefresh = ttoolbar_dorefresh;
	fnc->doevent = ttoolbar_doevent;
	fnc->domethod = ttoolbar_domethod;

	return 1;
}

struct tobj *ttoolbar_init(struct tobj *parent, int up)
{
	struct tobj *obj;
	struct ttoolbar *toolbar;
	char nomem[] = "ERROR: While allocating memory for ttoolbar.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(toolbar = calloc(1, sizeof(struct ttoolbar)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = toolbar;
		
		obj->x = 0;
		obj->y = (!up) ? t_screen_height : 0;
		obj->h = 1;
		obj->w = t_screen_width;

		/* setting default values */
		obj->fgcolor = TTOOLBARFG;
		obj->fgcolor2 = TTOOLBARSHINE;
		obj->bgcolor = TTOOLBARBG;
		obj->bgcolor2 = TTOOLBARBRACKETTS;
		obj->activeable=0;

		if (( ttoolbar_setup( obj , &obj->fnc))){
			tobj_init(parent, obj);
			return obj;
		} else
			return 0;
		
	}
}
