/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define WSEARCH_C_
#include "wsearch.h"

static int show;
static struct tobj  *line_search, *cmenu_search, *win_search;

void search_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;
	msearch_flush();
	linesearch_flush();
	tobj_domethod(win_search, TWIN_ACTIVATE_CHILD, (long)line_search);
	wsearch_show();	
}

int wsearch_service(struct tevent *e)
{
	char *name;
	long n;
	if (e->sender != line_search && e->sender != cmenu_search 
		&& e->sender !=	win_search && e->sender)
		return 0;
	
	switch (e->data) {
	case EVENT_TLIST_REJECT:
	case EVENT_TMENU_REJECT:
	case EVENT_TINPUT_REJECT:
		scrpeople_show();
		return 1;
	case EVENT_TINPUT_CHANGED:
		if ((name = (char *)txget(line_search, TINPUT_BUFFER_CURRENT)))
			msearch_additems(name);		
		return 1;

	case EVENT_TINPUT_ACCEPT:
		tget(cmenu_search, TLIST_LIST_LEN, &n);
		if (n > 1)
			tobj_domethod(win_search, 
				TWIN_ACTIVATE_CHILD, (long)cmenu_search);
		else
			msearch_service_m(0, 0);
		return 1;
	case K_UP:
		tobj_domethod(win_search, 
			TWIN_ACTIVATE_CHILD, (long)cmenu_search);
		tobj_domethod(cmenu_search, TLIST_ACTIVATE_PREV, 0);
		return 1;
	case K_DOWN:
		tobj_domethod(win_search, 
			TWIN_ACTIVATE_CHILD, (long)cmenu_search);
		tobj_domethod(cmenu_search, TLIST_ACTIVATE_NEXT, 0);
		return 1;
	}

	return 0;
}

static void msearch_service_m(struct tobj *obj, long data)
{
	char *str = (char *)txget(line_search, TINPUT_BUFFER_CURRENT);
	long n;

	(void) obj;
	(void) data;

	tget(cmenu_search, TLIST_ACTIVE, &n);

	if (n >= 0) {
		n = find_man_by_string(str, (int)n);
		scroll_men(n);
		mpeople_domethod(TLIST_ACTIVATE, n);	
	}
	scr_refresh();
}

static void msearch_additems(char *pattern)
{
	struct man *mj;
	char **jmp;
	struct drawer *dname;
	
	if (pattern == NULL || *pattern == '\0')
		return;
	
	pattern = strtolower(pattern);
	msearch_flush();

	for (mj = first_man; mj; mj = mj->next) {
		if (query_verify(pattern, mj->first) == 0)
			continue;
		dname = find_drawer_by_name(mj, menu_title);
		if (dname == 0)
			dname = mj->first;
		query("", dname->first);
	}

	for (jmp = query_list; jmp && *jmp; jmp++)
		if (!add_menuobj(cmenu_search, *jmp, msearch_service_m, 0))
			errexit("Error recived form tsys.\n");
	query_free();
	xfree(pattern);
}

int wsearch_create(struct tobj *rootwin)
{
	DEFINE_SIZES(Wsearch);
	if (!(win_search = twin_init(rootwin, x, y, w, h, translate("Search"))))
		return 0;
		
	if (!(msearch_flush()))
		return 0;

	if (!(linesearch_flush()))
		return 0;

	wsearch_hide();
	
	return 1;
}

void wsearch_show()
{
	tobj_show(win_search);
	show = 1;
}

void wsearch_hide()
{
	tobj_rhide(win_search);
	show = 0;
}

int wsearch_isshowing()
{
	return show;
}

static int linesearch_flush()
{
	int reinit = (line_search) ? 1 : 0;
	if (reinit)
		tobj_free(line_search);
	if (!(line_search = tinput_init(win_search, 1,
		get_h(Wsearch)-2, get_w(Wsearch)-2, "", 30))) {
		if (reinit)
			errexit("Cannot reinit line search.\n");
		return 0;
	}
	return 1;
}

static int msearch_flush(void)
{
	struct tcmenu_entry mtab[] = {{ TMENU_END , NULL , 0 }};
	int reinit = (cmenu_search) ? 1 : 0;
	if (reinit)
		tobj_free(cmenu_search);
	if (!(cmenu_search = tcmenu_init(win_search, 1, 1, 
		get_w(Wsearch)-2, get_h(Wsearch)-2, mtab))) {
		if (reinit)
			errexit("Cannot reinit cmenu search.\n");
		return 0;
	}
	return 1;
}

void wsearch_free()
{
	set_size(Wsearch, win_search);
	tobj_free(win_search);
	win_search = NULL;
	cmenu_search = NULL;
	line_search = NULL; 
}
